package com.perforce.hws.automation;

import com.perforce.hws.p4base.CloseableServer;
import com.perforce.hws.p4base.P4Methods;
import com.perforce.hws.p4base.ResultMap;
import com.perforce.hws.p4base.TemporaryClient;
import com.perforce.p4java.exception.P4JavaException;

import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import static java.nio.file.StandardOpenOption.APPEND;
import static java.nio.file.StandardOpenOption.CREATE;

/**
 * Sets up an environment using our mainline p4d build.
 */
public class BasicMainlineEnvironmentSetup implements IORunnable, FilesystemMethods,
        ManageP4dMethods, ManageHwsService, AutomationServerProvider,
        P4Methods, TemporaryClient, P4dInitConventions, EnvironmentCleanup {

    protected AutomationConfig config;

    public BasicMainlineEnvironmentSetup() {
        this.config = new AutomationConfig();
    }

    @Override
    public void run() throws IOException {
        try {
            createInstallationDir();

            startP4d();

            if (config.isEnableUnicodeP4d()) {
                resetToUnicode();
            }

            seedP4d();

            addP4dConfig();

            // If HWS is pre-installed and running, and we add p4d, we need to restart
            // HWS.
            // If it's not setup and already running, we probably are running locally
            // and we need to start it up.
            if (!isHWSRunning()){
                startHWS();
            } else {
                restartHwsService();
            }

        } catch (InterruptedException e) {
            System.err.println("Environment setup interrupted");
            // I'm not sure if we want to try to do anything here.
        } catch (P4JavaException e) {
            System.err.println("p4java failure: " + e.getMessage());
            e.printStackTrace();
        } catch (URISyntaxException e) {
            System.err.println("likely p4java failure: " + e.getMessage());
            e.printStackTrace();
        }
    }

    public static void main(String[] args) throws IOException {
        new BasicMainlineEnvironmentSetup().run();
    }

    private void seedP4d() throws P4JavaException, URISyntaxException, IOException {
        String superTicket = setSecuritySuperPassAndProtects();

        createCommonUsers(superTicket);
        createCommonDepots(config.getSuperUser(), superTicket);
        createCommonFiles(config.getSuperUser(), superTicket);

    }

    @Override
    public AutomationConfig config() {
        return config;
    }


}
