package com.perforce.hws.automation;

import com.perforce.hws.p4base.CloseableServer;
import com.perforce.hws.p4base.P4Methods;
import com.perforce.hws.p4base.ResultMap;
import com.perforce.hws.p4base.TemporaryClient;
import com.perforce.p4java.exception.P4JavaException;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.List;

/**
 * TODO: Spin up mock raymond and adjust the server configuration.
 */
public class CloudEnvironmentSetup implements IORunnable, ManageP4dMethods,
        FilesystemMethods, P4Methods, TemporaryClient, AutomationServerProvider,
        P4dInitConventions, ManageMockRaymond, ManageHwsService, EnvironmentCleanup {

    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(CloudEnvironmentSetup.class);

    private AutomationConfig config;

    public CloudEnvironmentSetup() {
        config = new AutomationConfig();
        config.setCloudEnabled(true);
    }

    @Override
    public AutomationConfig config() {
        return config;
    }

    @Override
    public void run() throws IOException {
        try {
            cleanUp();

            createInstallationDir();

            startP4d();

            resetToUnicode();

            seedP4d();

            startP4d();

            startMockRaymond();

            startHWS();

        } catch (InterruptedException e) {
            logger.error("Unknown interrupt, shutting down", e);
            throw new RuntimeException(e);
        } catch (P4JavaException e) {
            logger.error("P4Java Failure, shutting down", e);
            throw new RuntimeException(e);
        } catch (URISyntaxException e) {
            logger.warn("URI failure", e);
        }
    }

    public static void main(String[] args) throws IOException {
        new CloudEnvironmentSetup().run();
    }

    private void seedP4d() throws P4JavaException, URISyntaxException, IOException {
        String superTicket = setSecuritySuperPassAndProtects();

        createCommonUsers(superTicket);

        // Stage 3: Prepare "my_project" stream depot and main,dev streams
        //
        // Helix Cloud does not support classic Perforce depots, so we delete
        // the default depot.
        //
        try (CloseableServer s = obtain(null, config.getSuperUser(), superTicket)) {
            exec(s, "depot", "-d", "depot");

            List<ResultMap> results = exec(s, "depot", "-o", "my_project");
            ResultMap depot1 = results.get(0);
            depot1.put("Type", "stream");
            depot1.put("Description", "A cloud test depot");
            execInput(s, depot1, "depot", "-i");

            List<ResultMap> mainResults = exec(s,
                    "stream", "-o", "-t", "mainline", "//my_project/main");
            ResultMap mainStream = mainResults.get(0);
            mainStream.put("Name", "My~20Project");
            execInput(s, mainStream, "stream", "-i");

            List<ResultMap> devResults = exec(s,
                    "stream", "-o",
                    "-t", "development",
                    "-P", "//my_project/main",
                    "//my_project/dev");
            execInput(s, devResults.get(0), "stream", "-i");
        }

        // Stage 4: Submit some stuff
        //
        try (CloseableServer s = obtain(null, config.getSuperUser(), superTicket)) {
            submitMyProjectStreamData(s);
        }
    }
}
