package com.perforce.hws.automation;

import net.lingala.zip4j.core.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hws.deployment.PathConventions;
import com.perforce.hws.util.ProcessHelpers;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;

/**
 * Note: this typically needs to run with elevated privileges.
 */
public class DeployBinaryArchive implements ProcessHelpers, OsHelpers, PathConventions {

    static final Logger logger = LoggerFactory.getLogger(DeployBinaryArchive.class);

    public static void main(String args[]) {
        try {
            new DeployBinaryArchive(args[0]).run();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    String archivePath;

    public DeployBinaryArchive(String archivePath) {
        this.archivePath = archivePath;
    }

    public void run() throws Exception {
        if (isLinux()) {
            deployLinux();
        } else if (isOSX()) {
            deployOSX();
        } else if (isWindows()) {
            deployWindows();
        } else {
            throw new IllegalStateException("OS Not Yet Supported: " + osName());
        }
    }

    private void deployLinux() throws IOException, InterruptedException {
        prepUnix();
        logger.info(execOutOrFail("service", "helix-ws", "start"));
    }

    private void deployOSX() throws IOException, InterruptedException {
        prepUnix();
        logger.info(execOutOrFail("launchctl", "load", "-w", plistPath().toString()));
    }

    private void deployWindows() throws IOException, ZipException, InterruptedException {
        logger.info("deploying binary archive to C:\\helix-web-services");

        ZipFile zipFile = new ZipFile(archivePath);
        zipFile.extractAll("C:\\");

        logger.info(execOutOrFail("C:\\helix-web-services\\sbin\\setup-helix-ws.exe"));
        
        String scPath = Paths.get(System.getenv("windir"), "system32", "sc.exe").toString();
        logger.info(execOutOrFail(scPath, "start", "helix-ws"));
    }

    private void prepUnix() throws IOException, InterruptedException {
        logger.info("deploying binary archive to /opt/perforce");

        Files.createDirectories(Paths.get("/opt/perforce"));

        logger.info(execOutOrFail("tar", "-x", "-z", "-C", "/opt/perforce", "-f", archivePath));
        logger.info(execOutOrFail("/opt/perforce/helix-web-services/sbin/setup-helix-ws"));
    }

}
