package com.perforce.hws.automation;

import com.perforce.hws.p4base.P4Methods;
import com.perforce.hws.p4base.TemporaryClient;
import com.perforce.p4java.exception.P4JavaException;

import java.io.IOException;
import java.net.URISyntaxException;

/**
 * Created by tristan on 4/4/16.
 */
public class GitFusionEnvironmentSetup implements IORunnable, FilesystemMethods, ManageP4dMethods,
        ManageHwsService, AutomationServerProvider, P4Methods, TemporaryClient, P4dInitConventions,
        EnvironmentCleanup, GitFusionManagement {

    public static void main(String[] args) {
        try {
            new GitFusionEnvironmentSetup().run();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }

    protected AutomationConfig config;

    public GitFusionEnvironmentSetup() {
        config = new AutomationConfig();
        config.setP4DVersion(AutomationConfig.P4DVersion.P2016_1);
        config.setGitFusionEnabled(true);
    }

    @Override
    public AutomationConfig config() {
        return config;
    }

    @Override
    public void run() throws IOException {
        try {
            cleanUp();

            createInstallationDir();

            startP4d();

            resetToUnicode();

            seedP4d();

            setupGitFusion();

            startHWS();

        } catch (InterruptedException e) {
            System.err.println("Environment setup interrupted");
            // I'm not sure if we want to try to do anything here.
        } catch (P4JavaException e) {
            System.err.println("p4java failure: " + e.getMessage());
            e.printStackTrace();
        } catch (URISyntaxException e) {
            System.err.println("likely p4java failure: " + e.getMessage());
            e.printStackTrace();
        }
    }


    private void seedP4d() throws P4JavaException, URISyntaxException, IOException {
        String superTicket = setSecuritySuperPassAndProtects();

        createCommonUsers(superTicket);
        createCommonDepots(config.getSuperUser(), superTicket);
        createCommonFiles(config.getSuperUser(), superTicket);
    }

}
