package com.perforce.hws.automation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hws.util.StreamUtils;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;

/**
 * Created by tristan on 2/19/16.
 */
public interface OsHelpers extends StreamUtils {

    Logger logger = LoggerFactory.getLogger(OsHelpers.class);

    default boolean isOSX() {
        return osName().startsWith("Mac OS X");
    }

    default boolean isWindows() {
        return osName().startsWith("Windows");
    }

    default boolean isLinux() {
        return osName().startsWith("Linux");
    }

    default String osName() {
        return System.getProperty("os.name");
    }

    /**
     * @return e.g., "x86_64"
     */
    default String osArch() {
        return System.getProperty("os.arch");
    }

    default String nullPath() {
        if (isWindows()) {
            return "nul";
        } else {
            return "/dev/null";
        }
    }

    default boolean isCentOS() throws IOException {
        return hasRedhatRelease() && loadRedhatRelease().contains("CentOS");
    }

    default boolean hasRedhatRelease() {
        return Files.exists(Paths.get("/etc/redhat-release"));
    }

    default String loadRedhatRelease() throws IOException {
        return new String(Files.readAllBytes(Paths.get("/etc/redhat-release")));
    }

    default boolean isUbuntu() throws IOException, InterruptedException {
        return hasLsbRelease() && loadLsbRelease("-s -i").contains("Ubuntu");
    }

    default String loadUbuntuVersion() throws IOException, InterruptedException {
        return loadLsbRelease("-r -s").trim();
    }

    default boolean hasLsbRelease() throws IOException, InterruptedException {
        try {
            return Runtime.getRuntime().exec("lsb_release").waitFor() == 0;
        } catch (IOException e) {
//            logger.debug("Treating IOException for lsb_release as a " +
//                    "'command not found' scenario", e);
            return false;
        }
    }

    default String loadLsbRelease(String args) throws IOException, InterruptedException {
        Process p = Runtime.getRuntime().exec("lsb_release " + args);
        p.waitFor();
        if (p.exitValue() == 0) {
            return new String(readAll(p.getInputStream()));
        } else {
            throw new IllegalStateException("lsb_release failed with exit " +
                    "code: " + p.exitValue());
        }
    }
}
