package com.perforce.hws.automation;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.stream.Collectors;

import static com.perforce.hws.automation.FileUtil.appendEnvPath;
import static com.perforce.hws.automation.OsUtil.nullPath;
import static java.util.stream.Collectors.joining;

/**
 * Created by tjuricek on 1/11/16.
 */
public interface T4TestMethods extends ProjectLocations, UsesAutomationConfig {

    Logger logger = LoggerFactory.getLogger(T4TestMethods.class);

    default void runTests() throws IOException, InterruptedException {
        ProcessBuilder pb = new ProcessBuilder("perl",
                t4TestBin(),
                "--recurse",
                "helix-sync/shelve"
                // Just execute one sync test at a time
//                , "helix-sync/shelve/sync_add_submit.t"
                // Use verbose mode
//                , "-v"
                // Enable this to avoid deleting temporary directory
//                , "--keep-tmp"
//                // Enable this to save output to a local directory
//                , "--archive=/tmp/t4results.tar.gz"
        );

        pb.redirectInput(new File(nullPath()));
        pb.redirectErrorStream(true);
        pb.redirectOutput(ProcessBuilder.Redirect.INHERIT);

        pb.directory(new File(t4TestRoot()));

        String path = pb.environment().get("PATH");
        logger.debug("current path is {}", path);
        path = appendEnvPath(path, helixSyncBuildPath());
        logger.debug("updated path is {}", path);
        pb.environment().put("PATH", path);

        // This may increase the logging output of the HelixSync application
        pb.environment().put("HELIXSYNC_LOG_LEVEL", "3");

        pb.environment().put("HWS_URL", config().getLocalHws());

        logger.info("Launching t4 tests: {}", pb.command().stream().collect(joining(" ")));

        Process launched = pb.start();

        int exitCode = launched.waitFor();

        if (exitCode > 0) {
            throw new IllegalStateException("Tests failed with code " + exitCode);
        }
    }
}
